#!/usr/bin/env node
"use strict"
// fixed modules (global variable)
let dependencies = ["bootstrap","popper.js","rytm-webflow","rytm-helpers","rytm-wcag"]
// flexible modules (prompt)
const MODULES_PROMPT = ["gsap","imagesloaded"]

const chalk = require('chalk') // for colors in console.log
const mkdirp = require('mkdirp') // mkdir -p
const prompt = require('prompt') // prompt (user input)
const shell = require('shelljs') // to execute commands
const fs = require('fs') // file system
const __projectDir = process.cwd()


// welcome
const welcome = () => {
  console.log(chalk.magenta('									'))
  console.log(chalk.magenta('            #############    TM 	'))
  console.log(chalk.magenta('           ################ 		'))
  console.log(chalk.magenta('          ################# 		'))
  console.log(chalk.magenta('         ##### ########### 		'))
  console.log(chalk.magenta('        #####  ######## 			'))
  console.log(chalk.magenta('       #####    ##### 			'))
  console.log(chalk.magenta('      #####      ##### 			'))
  console.log(chalk.magenta('     #####        ##### 			'))
  console.log(chalk.magenta('    #####          ##### 			'))
  console.log(chalk.magenta('									'))
}
const goodbye = () => {
  console.log(chalk.magenta('									'))
  console.log(chalk.magenta('	All good.	One love! '))
  console.log(chalk.magenta('									'))
}

const isModuleInstalled = (name) => {
  return dependencies.indexOf(name) >= 0
}

const getPromptAttrs = () => {
  let props = [
    {
      description: 'Project name',
      name: 'project',
      type: 'string',
      validator: /^[a-zA-Z]+$/,
      warning: 'Project name is not valid, it can only contains letters',
      required: true
    }
  ]
  MODULES_PROMPT.forEach((moduleName) => {
    props.push({
      description: "Include module: " + moduleName,
      name: moduleName,
      type: 'boolean',
      default: false,
    })
  })
  return props
}
/**
 * copy file from template 
 */
const fromTemplate = (from, output, replace) => {
  let content = fs.readFileSync(from, 'utf8')
  replace.forEach((repl) => {
    content = content.split(repl.search).join(repl.replacement)
  })
  fs.writeFileSync(output, content);
}
// random token
const getRandomToken = (length) => {
  let result = '';
  let characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
  let charactersLength = characters.length;
  for ( let i = 0; i < length; i++ ) {
    result += characters.charAt(Math.floor(Math.random() * charactersLength));
  }
  return result;
}

const buildProject = () => {
  // prompt start
  prompt.start()
  // collect data
  prompt.get(getPromptAttrs(), (err, result) => {
    if (!result) {
      console.log(chalk.red('	Hate to see you leave :( '))
      return false;
    }
    const projectName = result.project
    let promptDependencies = []
    MODULES_PROMPT.forEach((moduleName) => {
      if (result[moduleName] == true) {
        promptDependencies.push(moduleName)
      }
    })
    dependencies = [...dependencies, ...promptDependencies]
    // # make directories
    mkdirp.sync('src/scripts');
    mkdirp.sync('src/scss');
    mkdirp.sync('public/local/config');
    mkdirp.sync('public/local/models');
    mkdirp.sync('public/local/images/favicons');
    mkdirp.sync('public/local/views/_boxes');
    mkdirp.sync('public/local/views/_boxes/svg');
    mkdirp.sync('public/local/views/item');
    mkdirp.sync('public/tmp/logs');
    mkdirp.sync('public/tmp/cache');
    mkdirp.sync('public/upload/thumb');
    fs.chmodSync('public/tmp', 0o777);
    fs.chmodSync('public/tmp/logs', 0o777);
    fs.chmodSync('public/tmp/cache', 0o777);
    fs.chmodSync('public/upload', 0o777);
    fs.chmodSync('public/upload/thumb', 0o777);
    // ## .gitignore ##
    fs.copyFileSync(
      __dirname + '/templates/_gitignore', 
      __projectDir + "/.gitignore"
    )
    // ## .htaccess ##
    fs.copyFileSync(
      __dirname + '/templates/_htaccess', 
      __projectDir + "/public/.htaccess"
    )
    // ## webpack.config.js ##
    fs.copyFileSync(
      __dirname + '/templates/webpack.config.js', 
      __projectDir + "/webpack.config.js"
    )
    // ## index.php ##
    fs.copyFileSync(
      __dirname + '/templates/index.php', 
      __projectDir + "/public/index.php"
    )
    // ## robots.txt ##
    fs.copyFileSync(
      __dirname + '/templates/robots.txt', 
      __projectDir + "/public/robots.txt"
    )
    // ## humans.txt ##
    fs.copyFileSync(
      __dirname + '/templates/humans.txt', 
      __projectDir + "/public/humans.txt"
    )
    // ## SRC files ##
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_bootstrap_migration.scss', 
      __projectDir + "/src/scss/_bootstrap_migration.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_buttons.scss', 
      __projectDir + "/src/scss/_buttons.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_common.scss', 
      __projectDir + "/src/scss/_common.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_typography.scss', 
      __projectDir + "/src/scss/_typography.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_utilities.scss', 
      __projectDir + "/src/scss/_utilities.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_variables.scss', 
      __projectDir + "/src/scss/_variables.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/__variables_sample.scss', 
      __projectDir + "/src/scss/__variables_sample.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/_wcag_common.scss', 
      __projectDir + "/src/scss/_wcag_common.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scss/style.scss', 
      __projectDir + "/src/scss/style.scss"
    )
    fs.copyFileSync(
      __dirname + '/templates/src/scripts/app.js', 
      __projectDir + "/src/scripts/app.js"
    )
    // ## package.json ##
    fromTemplate(
      __dirname + '/templates/package.json', 
      __projectDir + '/package.json', 
      [
        {
          search: "<%= project_name %>",
          replacement: projectName
        }
      ]
    )
    // ## rytm-webflow dependendent 
    if (isModuleInstalled("rytm-webflow")) {
      mkdirp.sync('src/scripts/controllers');
      mkdirp.sync('src/scripts/shared');
      mkdirp.sync('src/scripts/views');
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/shared/common.js', 
        __projectDir + "/src/scripts/shared/common.js"
      )
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/shared/cookies.js', 
        __projectDir + "/src/scripts/shared/cookies.js"
      )
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/shared/linksFix.js', 
        __projectDir + "/src/scripts/shared/linksFix.js"
      )
      // aswap
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/controllers/Controller.js', 
        __projectDir + "/src/scripts/controllers/Controller.js"
      )
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/views/View.js', 
        __projectDir + "/src/scripts/views/View.js"
      )
      fs.copyFileSync(
        __dirname + '/templates/src/scripts/aswap.js', 
        __projectDir + "/src/scripts/aswap.js"
      )
    }
    // ### LOCAL ###
    fs.copyFileSync(
      __dirname + '/templates/local/models/item.php', 
      __projectDir + "/public/local/models/item.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/views/index.php', 
      __projectDir + "/public/local/views/index.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/views/item/item_full.php', 
      __projectDir + "/public/local/views/item/item_full.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/views/item/item_image_lg.php', 
      __projectDir + "/public/local/views/item/item_image_lg.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/views/_boxes/head_favicons.php', 
      __projectDir + "/public/local/views/_boxes/head_favicons.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/views/_boxes/header_wcag.php', 
      __projectDir + "/public/local/views/_boxes/header_wcag.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/views/_boxes/pagination.php', 
      __projectDir + "/public/local/views/_boxes/pagination.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/views/_boxes/svg/rtm.php', 
      __projectDir + "/public/local/views/_boxes/svg/rtm.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/config/setup.php', 
      __projectDir + "/public/local/config/setup.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/config/local-sample.php', 
      __projectDir + "/public/local/config/local-sample.php"
    )
    fs.copyFileSync(
      __dirname + '/templates/local/config/imageconfig.php', 
      __projectDir + "/public/local/config/imageconfig.php"
    )
    // ## config/local.php ##
    fromTemplate(
      __dirname + '/templates/local/config/local.php', 
      __projectDir + '/public/local/config/local.php', 
      [
        {
          search: "<%= devdbname %>",
          replacement: projectName
        },
        {
          search: "<%= devdbpass %>",
          replacement: projectName
        },
        {
          search: "<%= devtoken %>",
          replacement: getRandomToken(40)
        }
      ]
    )

    // ### npm install
    shell.exec("npm install")
    shell.exec("npm install --save " + dependencies.join(" "))
    shell.exec("npm run initmodules")
    shell.exec("npm run build")
    goodbye()

  }) 

}

// welcome msg
welcome()
// build project
buildProject()